Package: tkWidgets
Version: 1.90.0
Title: R based tk widgets
Author: J. Zhang <jzhang@jimmy.harvard.edu>
Maintainer: J. Zhang <jzhang@jimmy.harvard.edu>
Depends: R (>= 2.0.0), methods, widgetTools (>= 1.1.7), DynDoc (>=
        1.3.0), tools
Suggests: Biobase, hgu95av2
Description: Widgets to provide user interfaces. tcltk should have
        been installed for the widgets to run.
LazyLoad: yes
biocViews: Infrastructure
Keyword: graphics
License: Artistic-2.0
git_url: https://git.bioconductor.org/packages/tkWidgets
git_branch: RELEASE_3_23
git_last_commit: 82dbb93
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:11:31 UTC; biocbuild
